

import java.awt.EventQueue;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import java.awt.BorderLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import java.awt.Dimension;
import javax.swing.JTable;
import javax.swing.tree.DefaultMutableTreeNode;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.Cloud.*;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTable lvLog;
	private JTable lvObjects;
    DefaultMutableTreeNode rootNode;
    
    private TElFileDataStorage m_DataStorage = new TElFileDataStorage();
    private ProgressDialog m_CurrProgress = null;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				exitClick();
			}
		});	
		
		setTitle("Secure Windows Azure Data Storage demo");
		setBounds(100, 100, 685, 465);
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		JMenu mnStorage = new JMenu("Storage");
		menuBar.add(mnStorage);
		
		JMenuItem mntmOpen = new JMenuItem("Open");
		mntmOpen.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				DoOpen();
			}
		});
		mnStorage.add(mntmOpen);
			
		JSeparator separator = new JSeparator();
		mnStorage.add(separator);
		
		JMenuItem mntmExit = new JMenuItem("Exit");
		mntmExit.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				dispose();
			}
		});
		mnStorage.add(mntmExit);
		
		JMenu mnHelp = new JMenu("Help");
		menuBar.add(mnHelp);
		
		JMenuItem mntmAbout = new JMenuItem("About");
		mntmAbout.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				aboutClick();
			}
		});
		mnHelp.add(mntmAbout);
		
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		getContentPane().add(toolBar, BorderLayout.NORTH);
		
		JButton btnOpen = new JButton("");
		btnOpen.setIcon(getIcon("open"));
		btnOpen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DoOpen();
			}
		});
		toolBar.add(btnOpen);
		
		JButton btnRead = new JButton("");
		btnRead.setIcon(getIcon("read"));
		btnRead.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DoGet();
			}
		});
		toolBar.add(btnRead);
		
		JButton btnWrite = new JButton("");
		btnWrite.setIcon(getIcon("write"));		
		btnWrite.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DoPut();
			}
		});
		toolBar.add(btnWrite);
		
		JButton btnInfo = new JButton("");
		btnInfo.setIcon(getIcon("info"));
		btnInfo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DoInfo();
			}
		});
		toolBar.add(btnInfo);
		
		JPanel panel = new JPanel();
		panel.setPreferredSize(new Dimension(10, 80));
		getContentPane().add(panel, BorderLayout.SOUTH);
		panel.setLayout(new BorderLayout(0, 0));
		
		JScrollPane scrollPane = new JScrollPane();
		panel.add(scrollPane, BorderLayout.CENTER);
		
		lvLog = new JTable(new LogTableModel());
		lvLog.setFillsViewportHeight(true);
		scrollPane.setViewportView(lvLog);
		
		rootNode = new DefaultMutableTreeNode("All Buckets");
		
		JScrollPane scrollPane_2 = new JScrollPane();
		getContentPane().add(scrollPane_2, BorderLayout.CENTER);
		
		lvObjects = new JTable(new ObjectsTableModel());
		lvObjects.setFillsViewportHeight(true);
		scrollPane_2.setViewportView(lvObjects);

		init();
	}

	protected void exitClick() {
		m_DataStorage.Free();
		System.exit(0);
	}

	private Icon getIcon(String name) {
		URL resPath = getClass().getResource(name + ".ico");
		if (resPath == null) return new ImageIcon();
		return new ImageIcon(resPath);
	}

	private void init() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");

        SBDataStorage.dataStorageEncodingHandlersFactory().registerHandler(new TElZlibDataStorageEncodingHandler().getClass());

        m_DataStorage.setOnSecurityHandlerCreated(new TSBFDSSecurityHandlerCreatedEvent(m_DataStorage_OnSecurityHandlerCreated));
        m_DataStorage.setOnProgress(new TSBDataStorageProgressEvent(m_DataStorage_OnProgress));
	}

	TSBFDSSecurityHandlerCreatedEvent.Callback m_DataStorage_OnSecurityHandlerCreated = new TSBFDSSecurityHandlerCreatedEvent.Callback() {
		
		public void tsbfdsSecurityHandlerCreatedEventCallback(TObject sender,
				TElCustomDataStorageObject obj, TObject data,
				TElCustomDataStorageSecurityHandler handler) {
            if (handler instanceof TElDefaultDataStorageSecurityHandler)
            {
                UnprotHandlerPropsDialog dlg = new UnprotHandlerPropsDialog();
                dlg.init((TElDefaultDataStorageSecurityHandler)handler);
                dlg.setModal(true);
                dlg.setVisible(true);
            }
            else
            {
                showMessage("Unknown security handler: " + handler.getDescription());
            }
		}
	};
	
	TSBDataStorageProgressEvent.Callback m_DataStorage_OnProgress = new TSBDataStorageProgressEvent.Callback() {
		
		public void tsbDataStorageProgressEventCallback(TObject sender,
				TSBDataStorageOperation op, long total, long current, TSBBoolean cancel) {
            if (m_CurrProgress != null){
                cancel.value = m_CurrProgress.update(total, current);
            }else
            	cancel.value = false;

		}
	};
	
	protected void aboutClick() {
		AboutDialog dlg = new AboutDialog();
		dlg.setModal(true);
		dlg.setVisible(true);
	}

	public void showMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	public void showErrorMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
	}
	
    private void DoOpen()
    {
    	String folderName = getFileName();
        if (folderName.length() > 0)
        {
            m_DataStorage.setDirectory(folderName);
            RefreshObjectList();
        }
    }
    
	protected String getFileName() {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);		
		
		int returnVal = fc.showOpenDialog(this);

        if (returnVal == JFileChooser.APPROVE_OPTION)
            return fc.getSelectedFile().getAbsolutePath();
        
        return "";
	}
    
	private void clearObjectItems() {
		((ObjectsTableModel)lvObjects.getModel()).clear();
	}

    private void RefreshObjectList()
    {
    	ObjectsTableModel omodel = ((ObjectsTableModel)lvObjects.getModel());
        for (int i = 0; i < omodel.getRowCount(); i++)
        {
            TElCustomDataStorageObject obj = omodel.getTag(i);
            m_DataStorage.releaseObject(obj);
        }
        clearObjectItems();

        TElDataStorageObjectList list = new TElDataStorageObjectList();
        m_DataStorage.list(list);
        
        for (int i = 0; i < list.getCount(); i++)
        {
        	TElCustomDataStorageObject item = list.getObject(i);
        	((ObjectsTableModel)lvObjects.getModel()).addItem(item);
        }
    }
    
    private void DoGet()
    {
        if (lvObjects.getSelectedRowCount() > 0)
        {
        	ObjectsTableModel omodel = ((ObjectsTableModel)lvObjects.getModel());
            int selectedRow = lvObjects.getSelectedRow();
            final TElFileDataStorageObject tag = omodel.getTag(selectedRow);
            
            final DownloadPropsDialog dlg = new DownloadPropsDialog();
            dlg.Prepare(tag);
            dlg.setModal(true);
            dlg.setVisible(true);
            
            if (dlg.isOK())
            {
                java.lang.Thread t = new java.lang.Thread(new Runnable() {
					
					public void run() {
		                try
		                {
		                    TElFileStream f = new TElFileStream(dlg.getDestPath(), "rw", true);
		                    try
		                    {
		                        ShowProgressForm("Downloading object " + tag.getName());
		                        try
		                        {
		                            if (dlg.getPartialDownload())
		                            {
		                                TSBLong Read = new TSBLong();
		                                m_DataStorage.readBlock(tag, f, dlg.getStartOffset(), dlg.getSizeEx(), Read);
		                                if (Read.value != dlg.getSizeEx())
		                                {
		                                    showMessage("Read " + Read.value + " bytes of requested " + dlg.getSizeEx());
		                                }
		                            }
		                            else
		                            {
		                                m_DataStorage.readObject((TElCustomDataStorageObject)tag, f);
		                            }
		                        }
		                        finally
		                        {
		                            HideProgressForm();
		                        }
		                    }
		                    finally
		                    {
		                        f.Free();
		                    }
		                }
		                catch (Exception e)
		                {
		                	e.printStackTrace();
		                    showErrorMessage("Error: " + e.getMessage());
		                }						
					}
				});
            	
                t.start();
            }
        }
    }
    
    private void DoPut()
    {    	
        final ProtHandlerPropsDialog dlg = new ProtHandlerPropsDialog();
        dlg.Init();
        dlg.setModal(true);
        dlg.setVisible(true);
        if (dlg.isOK())
        {
            java.lang.Thread t = new java.lang.Thread(new Runnable() {
                public void run() { 
        	
		            TElDefaultDataStorageSecurityHandler handler = null;
		            if (dlg.getProtect())
		            {
		                handler = new TElDefaultDataStorageSecurityHandler();
		                TElMemoryCertStorage encryptionCerts = new TElMemoryCertStorage();
		                encryptionCerts.add(dlg.getCert(), false);
		                handler.setEncryptionCertificates(encryptionCerts);
		                handler.setKeyEncryptionAlgorithm(SBConstants.SB_ALGORITHM_CNT_AES128);
		                handler.setDataEncryptionAlgorithm(SBConstants.SB_ALGORITHM_CNT_AES128);
		            }
		            
		            try
		            {
		                TElFileStream f = new TElFileStream(dlg.getSourceFile(), "rw", true);
		                try
		                {
		                    ShowProgressForm("Uploading object " + dlg.getObjectName());
		                    try
		                    {
		                        if (handler != null)
		                        {
		                            if (dlg.getIntegrityProtection() == 0)
		                            {
		                                handler.setAuthInfoType(TSBDSDataAuthInfoType.aitNone);
		                            }
		                            else
		                            {
		                                if (dlg.getIntegrityProtection() == 1)
		                                {
		                                    handler.setAuthInfoType(TSBDSDataAuthInfoType.aitBasic);
		                                }
		                                else
		                                {
		                                    handler.setAuthInfoType(TSBDSDataAuthInfoType.aitExtended);
		                                    handler.setAuthDigestLocation(TSBDSDigestLocation.dlEmbedded);
		                                    handler.setAuthBlockSize(1024); // per-Kb hashing
		                                    handler.setAuthHashSize(10); // truncating a half of SHA1 result
		                                }
		                                handler.setAuthDigestAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
		                            }
		                        }
		                        
	
	                            if (dlg.getStartOffset() == 0)
	                            {
	                                m_DataStorage.writeObject(dlg.getObjectName(), f, handler);
	                            }
	                            else
	                            {
	                                TSBLong written = new TSBLong();
	                                m_DataStorage.writeBlock(dlg.getObjectName(), f, (TElCustomDataStorageSecurityHandler)handler, dlg.getStartOffset(), written);
	                                if (written.value != f.getLength())
	                                {
	                                    showMessage("Written " + written.value + " bytes of requested " + f.getLength());
	                                }
	                            }

		                    }
		                    finally
		                    {
		                        HideProgressForm();
		                    }
		                }
		                finally
		                {
		                    f.Free();
		                }
		            }
		            catch (Exception e)
		            {
		                showErrorMessage("Object write error: " + e.getMessage());
		            }
		            RefreshObjectList();
                }
            });
            
            t.start();
            
        }
    }
    
    private String AddNameValuePair(String name, String value, String str)
    {
        return str + name + ": " + value + "\r\n";
    }

    private void DoInfo()
    {
        if (lvObjects.getSelectedRowCount() > 0)
        {
            int selectedRow = lvObjects.getSelectedRow();
            TElFileDataStorageObject tag = ((ObjectsTableModel)lvObjects.getModel()).getTag(selectedRow);
            TElCustomDataStorageSecurityHandler handler = null;
            try
            {
                handler = m_DataStorage.getProtectionInfo(tag);
            }
            catch (Exception ex)
            {
                ;
            }
            if (handler instanceof TElDefaultDataStorageSecurityHandler)
            {
                TElDefaultDataStorageSecurityHandler defHandler = (TElDefaultDataStorageSecurityHandler)handler;
                String S = "";
                S = AddNameValuePair("Data encryption algorithm", SBConstants.getAlgorithmNameByAlgorithm(defHandler.getDataEncryptionAlgorithm()), S);
                S = AddNameValuePair("Key encryption algorithm", SBConstants.getAlgorithmNameByAlgorithm(defHandler.getKeyEncryptionAlgorithm()), S);
                if (defHandler.getAuthInfoType() == TSBDSDataAuthInfoType.aitNone)
                {
                    S = AddNameValuePair("Data authentication info", "N/A", S);
                }
                else
                {
                    if (defHandler.getAuthInfoType() == TSBDSDataAuthInfoType.aitBasic)
                    {
                        S = AddNameValuePair("Data authentication info", "Basic", S);
                        S = AddNameValuePair("Data authentication algorithm", SBConstants.getAlgorithmNameByAlgorithm(defHandler.getAuthDigestAlgorithm()), S);
                    }
                    else if (defHandler.getAuthInfoType() == TSBDSDataAuthInfoType.aitExtended)
                    {
                        S = AddNameValuePair("Data authentication info", "Extended", S);
                        S = AddNameValuePair("Data authentication algorithm", SBConstants.getAlgorithmNameByAlgorithm(defHandler.getAuthDigestAlgorithm()), S);
                        String DL = "";
                        if (defHandler.getAuthDigestLocation() == TSBDSDigestLocation.dlLocal)
                        	DL = "Local";
                        if (defHandler.getAuthDigestLocation() == TSBDSDigestLocation.dlAppended)
                            DL = "Appended";
                        if (defHandler.getAuthDigestLocation() == TSBDSDigestLocation.dlEmbedded)
                            DL = "Embedded";

                        S = AddNameValuePair("Data authentication digest location", DL, S);
                        S = AddNameValuePair("Data authentication block size", defHandler.getAuthBlockSize() + "", S);
                        S = AddNameValuePair("Data authentication hash size", defHandler.getAuthHashSize() + "", S);
                    }
                }
                for (int i = 0; i < defHandler.getCertIDCount(); i++)
                {
                    S = AddNameValuePair("Recipient #" + (i + 1), "Issuer: " +
                    defHandler.getCertID(i).getIssuer().saveToDNString() + ", SN: " + SBUtils.binaryToString(defHandler.getCertID(i).getSerialNumber()), S);
                }
                showMessage(S);
            }
            else if (handler == null)
            {
                showMessage("The object is not protected");
            }
            else
            {
                showMessage("Unknown security handler");
            }
        }
    }  
    
    private void ShowProgressForm(String caption)
    {
        m_CurrProgress = new ProgressDialog();
        m_CurrProgress.Prepare(caption);
        m_CurrProgress.setVisible(true);
    }

    private void HideProgressForm()
    {
        if (m_CurrProgress != null) 
        {
            m_CurrProgress.setVisible(false);
            m_CurrProgress.dispose();
            m_CurrProgress = null;
        }
    }   
}