package com.eldos;

import SecureBlackbox.Base.*;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.XML.TElXMLDOMDocument;
import SecureBlackbox.XMLSecurity.TElXMLKeyInfoX509Data;
import SecureBlackbox.XMLSecurity.TElXMLSigner;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;

@SuppressWarnings("serial")
public class ResultServlet extends HttpServlet {
	
	private static final String URL = "<a href=\"/dcweb/result?name=%s\">download</a>";

    private Cache cache = Cache.getInstance();

	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String sessionID = req.getParameter("sessionid");
		
		if ((sessionID == null) || (req.getContentLength() == 0)) {
			resp.sendError(400);
			return;
		}
		if ( ! "application/xml".equals(req.getContentType())) {
			resp.sendError(415);
			return;
		}
		
		InputStream is = req.getInputStream();
		byte[] signature = SBUtils.streamToBuffer(is, 0);
		
		TElMemoryStream result = finishSigning(sessionID, signature);
        cache.put(sessionID, result.getBuffer());

        gotoResult(req, resp, "Signed file: " + String.format(URL, sessionID));
	}
	
	public void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		String name = req.getParameter("name");
		
		if (name != null) { // return signed file
			byte[] data = cache.get(name);
			if (data == null) {
				gotoResult(req, resp, "No signed file in cache");
				return;
			}
			
			resp.setContentType("application/xml");
			resp.setHeader("Content-Disposition", "filename=sample-" + name + ".xml");
			
			resp.getOutputStream().write(data);
			resp.getOutputStream().flush();
			
		} else { // finish signing
			
			String sessionID = req.getParameter("sessionid");
			
			if (sessionID == null) {
				gotoResult(req, resp, "Empty session");
				return;
			}
				
			String data = req.getParameter("data");
			if (data != null) {
				byte[] signature = SBEncoding.base64DecodeArray(data);

				TElMemoryStream result = finishSigning(sessionID, signature);
                cache.put(sessionID, result.getBuffer());
			}

            gotoResult(req, resp, "Signed file: " + String.format(URL, sessionID));
		}
	}
	
	private void gotoResult(HttpServletRequest req, HttpServletResponse resp, String respStr) throws IOException {
		req.setAttribute("labelResult", respStr);
		try {
			req.getRequestDispatcher("/result.jsp").forward(req, resp);
		} catch (ServletException e) {
			resp.sendError(500);
		}
	}

	private TElMemoryStream finishSigning(String sessionID, byte[] signature) {
		TElDCAsyncState state = new TElDCAsyncState();
        TElMemoryStream input = new TElMemoryStream(signature, 0, signature.length);
        state.loadFromStream(input, SBDCXMLEnc.dcxmlEncoding());

        byte[] docBin = cache.get(sessionID);
        TElMemoryStream result = new TElMemoryStream(docBin, 0, docBin.length);

        TElXMLDOMDocument doc = new TElXMLDOMDocument();
        doc.loadFromStream(result);

        TElXMLSigner signer = new TElXMLSigner();
        TElXMLKeyInfoX509Data x509Data = new TElXMLKeyInfoX509Data(true);

        TElDCBaseMessage msg = state.findMessageByType(TElDCOperationResponseMessage.class);
        if (msg != null) {
            byte[] buf = ((TElDCOperationResponseMessage)msg).getKeysRDN().getFirstValueByOID(SBDCPKIConstants.SB_OID_DC_SIGNING_CERTIFICATE.Data);
            TElX509Certificate cert = new TElX509Certificate();
            if (cert.loadFromBufferAuto(buf, 0, buf.length, "") == 0) {
                x509Data.setCertificate(cert);
                signer.setKeyData(x509Data);
                signer.setIncludeKey(true);
            }

            signer.completeAsyncSign(doc, state);
        }

        result.setLength(0);
        doc.saveToStream(result);
        result.setPosition(0);

        return result;
	}
}
