import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import SecureBlackbox.FTPSServer.TElSimpleFTPSServerSessionThread;

public class ConnTableModel extends AbstractTableModel {

	private static final long serialVersionUID = 1L;
	public static final int HOST_INDEX = 0;
	public static final int USER_INDEX = 1;
	public static final int CURDIR_INDEX = 2;
	public static final int CUROP_INDEX = 3;
	public static final int PROGR_INDEX = 4;
	
	protected String columnNames[] = { "Remote host", "User", "Current directory", "Current operation", "Progress:" };
	protected Vector<ConnDataRow> dataVector = new Vector<ConnDataRow>();
	
 	public String getColumnName(int column) {
        return columnNames[column];
    }
 	
	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return dataVector.size();
	}

	public Object getValueAt(int row, int col) {
		ConnDataRow record = dataVector.get(row);
		switch(col){
			case HOST_INDEX:
				return record.Host;
			case USER_INDEX:
				return record.User;
			case CURDIR_INDEX:
				return record.CurDir;
			case CUROP_INDEX:
				return record.CurOp;
			case PROGR_INDEX:
				return record.Progress;
		}
		return new Object();
	}

	public TElSimpleFTPSServerSessionThread getSelectedRowTag(int row) {
		return dataVector.get(row).Tag;
	}

	public void clear() {
		dataVector.clear();
		fireTableStructureChanged();
	}

	public void remove(TElSimpleFTPSServerSessionThread session) 
	{		
		for (int i = 0; i < dataVector.size(); i++) 
		{
			if (dataVector.get(i).Tag == session)
			{
				dataVector.remove(i);
				fireTableDataChanged();
				return;
			}
		}		
	}

	public void change(TElSimpleFTPSServerSessionThread session, String host,
			String username, String homeDirectory, String getLastCommand,
			String progress) {
		for (int i = 0; i < dataVector.size(); i++)
		{
			ConnDataRow row = dataVector.get(i);
			if (row.Tag == session)
			{
				row.Host = host;
				row.User = username;
				row.CurDir = homeDirectory;
				row.CurOp = getLastCommand;
				row.Progress = progress;
				fireTableRowsUpdated(i, i);
				
				return;
			}
		}

		dataVector.add(new ConnDataRow(session, host, username, homeDirectory, getLastCommand, progress));
		fireTableRowsInserted(dataVector.size(), dataVector.size());		
	}

	public void add(TElSimpleFTPSServerSessionThread session) {
		dataVector.add(new ConnDataRow(session));
		fireTableRowsInserted(dataVector.size(), dataVector.size());
	} 
}