import java.io.File;

import java.util.Date;
import java.util.Vector;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;

public class DiskFileSystemAdapter extends TElCustomFileSystemAdapter implements TagEvent {
	final String PathSeparator = "/";

	private TagEvent m_listener;

	public DiskFileSystemAdapter(TagEvent e) {
		m_listener = e;
	}

	@Override 
	public Object clone()
	{
  		DiskFileSystemAdapter res = new DiskFileSystemAdapter(null);
  		res.setCaseSensitive(FCaseSensitive);
  		res.setBasePath(FBasePath);
  		res.setPathSeparator(FPathSeparator);
  		res.setPathSeparator2(FPathSeparator2);
  		
  		return res;  
	}
	
	@Override
	protected boolean collectDirectoryEntries(String Path, String SubPath, String RegexValue,
	        TElStringList MaskList, boolean Recursive, ArrayList List)
	    {
		
	        String RegexProcessor = null;
	        TElVFSEntryInformation Info;
	        String[] Dirs;
	        File[] FSInfos;
	        int TotalMasks;
	        String TmpPath, Mask = "*.*";

	        if (RegexValue != "")
	        {
	            RegexProcessor = new String(RegexValue);
	            Mask = "*.*";
	            TotalMasks = 1;
	        }
	        else
	        {
	            TotalMasks = MaskList.getCount();
	        }

	        for (int i = 0; i < TotalMasks; i++)
	        {
	            if (RegexValue == "")
	                Mask = (String) MaskList.get(i);

	            try
	            {
	                FSInfos = GetFileSystemInfos(Path, Mask);

	                for (int j = 0; j < FSInfos.length; j++)
	                {
	                    if (((RegexValue == "") || ((RegexProcessor != null) && (RegexProcessor.matches(FSInfos[j].getName())))) &&
	                        (FSInfos[j].getName() != ".") && (FSInfos[j].getName() != ".."))
	                    {
	                        Info = new TElVFSEntryInformation();
	                        CopyEntryInfo(FSInfos[j], Info);
	                        List.add(Info);

	                        if (SubPath != "")
	                            Info.setFullName(SubPath + PathSeparator + Info.getName());
	                        else
	                            Info.setFullName(Info.getName());
	                    }
	                }
	            }
	            catch (Exception ex)
	            {
	                return false;
	            }
	        }

	        if (Recursive)
	        {
	            Dirs = GetDirectories(Path);
	            
	            for (int i = 0; i < Dirs.length; i++)
	            {
	                if ((Dirs[i] != ".") && (Dirs[i] != ".."))
	                {
	                    TmpPath = Dirs[i];
	                    
	                    collectDirectoryEntries(Dirs[i], TmpPath, RegexValue,
	                        MaskList, Recursive, List);
	                }
	            }
	        }
	        
	        return true;
	}

	String[] GetDirectories(String path) {
		Vector<String> result = new Vector<String>();
		File items[] = new File(path).listFiles();
		 for (int i = 0; i < items.length; i++) {
             if (items[i].isDirectory()) {
            	 result.add(items[i].getAbsolutePath());
             }
		 }
		 return (String[]) result.toArray(new String[result.size()]);
	}
	
	String[] GetFiles(String path) {
		Vector<String> result = new Vector<String>();
		File items[] = new File(path).listFiles();
		 for (int i = 0; i < items.length; i++) {
             if (items[i].isFile()) {
            	 result.add(items[i].getAbsolutePath());
             }
		 }
		 return (String[]) result.toArray(new String[result.size()]);
	}

    protected void CopyEntryInfo(File FSInfo, TElVFSEntryInformation EntryInfo)
    {
        EntryInfo.setName(FSInfo.getName());

        if (FSInfo.isDirectory())
            EntryInfo.setFileType(TSBFileType.sftDirectory);
        else
            EntryInfo.setFileType(TSBFileType.sftFile);
        
        Date modified = new Date(FSInfo.lastModified());
        EntryInfo.setDateAccessed(modified);
        EntryInfo.setDateCreated(modified);
        EntryInfo.setDateModified(modified);
        EntryInfo.setAccessMode(0);
        EntryInfo.setOwnerName("");

        if (FSInfo.isFile())
            EntryInfo.setSize(FSInfo.length());
        else
            EntryInfo.setSize(0);
    }

	private File[] GetFileSystemInfos(String path, String mask) {
		String [] dirs = GetDirectories(path);
		String [] files = GetFiles(path);
		
		Vector<File> result = new Vector<File>();
		
		for (int i = 0; i < dirs.length; i++){
			if (SBStrUtils.filenameMatchesMask(dirs[i], mask, FCaseSensitive)) {
				result.add(new File(dirs[i]));
			}
		}
		
		for(int i = 0; i < files.length; i++) {
			if (SBStrUtils.filenameMatchesMask(files[i], mask, FCaseSensitive)) {
				result.add(new File(files[i]));
			}
		}
		
		return (File[]) result.toArray(new File[result.size()]);
	}

	@Override
	public TSBInteger deleteFileTag(String path, String tagName) {
		TSBInteger res = new TSBInteger();
		OnDeleteFileTag(this, path, tagName, res);
   		return res;
	}

	@Override
	public boolean directoryExists(String path) {
		if (!new File(path).isDirectory()) return false;
		return new File(adjustPath(path)).exists();
	}

	@Override
	public int directoryMake(String path) {
		return (new File(adjustPath(path)).mkdir() ? 1 : 0);
	}

	@Override
	public int directoryRemove(String path) {
		return (new File(adjustPath(path)).delete() ? 1 : 0);
	}

	@Override
	protected void doCloseFile(TObject FileHandle) {
        if (FileHandle != null)
        {
            if (FileHandle instanceof FTPSFileStream)
            	((FTPSFileStream)FileHandle).Destroy();
        }		
	}

	@Override
	protected int doOpenFile(String path, int openMode, TSBFileShareMode shareMode,
			TElVFSOutputFileHandle fileHandle) {
		String RealPath = adjustPath(path);
        
		try
        {
			fileHandle.Handle = new FTPSFileStream(RealPath, openMode, shareMode);           
        }
        catch (Exception ex)
        {
            return SBCustomFSAdapter.SB_VFS_ERROR_ACCESS_DENIED;
        }

        return 0;
	}

	@Override
	public int fileDelete(String path) {
        if ( ! new File(path).exists())
            return SBCustomFSAdapter.SB_VFS_ERROR_FILE_NOT_FOUND;

        try {
            new File(adjustPath(path)).delete();
        }
        catch (Exception ex) {
            return SBCustomFSAdapter.SB_VFS_ERROR_ACCESS_DENIED;
        }

        return 0;
	}

	@Override
	public boolean fileExists(String path) {
		return new File(adjustPath(path)).exists();
	}

	@Override
	public long fileGetPosition(TObject fileHandle) {
        try
        {
            if (fileHandle instanceof FTPSFileStream)
            {
                return ((FTPSFileStream)fileHandle).GetPosition();
            }
            else
                return -1;
        }
        catch (Exception ex)
        {
            return -1;
        }
	}

	@Override
	public long fileGetSize(TObject fileHandle) {
        try
        {
            if (fileHandle instanceof FTPSFileStream)
            {
                return ((FTPSFileStream)fileHandle).GetSize();
            }
            else
                return -1;
        }
        catch (Exception ex)
        {
            return -1;
        }
	}

	@Override
	public int fileMove(String from, String to) {
        try
        {
            new File(from).renameTo(new File(adjustPath(to)));
        }
        catch (Exception ex)
        {
            return SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
        }

        return 0;
	}

	@Override
	public int fileRead(TObject handle, byte[] buffer, int index, int size) {
        try
        {
            if (handle instanceof FTPSFileStream)
            {
                return ((FTPSFileStream)handle).read(buffer, index, size);
            }
            else
            {
                return 0;
            }
        }
        catch (Exception ex)
        {
            return 0;
        }
    }

	@Override
	public long fileSeek(TObject FileHandle, long Position, TSBFileSeekOrigin Origin) {

        try
        {
            if (FileHandle instanceof FTPSFileStream)
            {
                ((FTPSFileStream)FileHandle).setPosition(Position);
                return ((FTPSFileStream)FileHandle).GetPosition();
            }
            else
            {
                return 0;
            }
        }
        catch (Exception ex)
        {
            return 0;
        }
	}

	@Override
	public void fileSetSize(TObject FileHandle, long NewSize) {
        try
        {
            if (FileHandle instanceof FTPSFileStream)
            {
            	((FTPSFileStream)FileHandle).SetLength(NewSize);
            }
        }
        catch (Exception ex)
        {

        }		
	}

	@Override
	public int fileWrite(TObject FileHandle, byte[] Buffer, int Index, int Size) {
        try
        {
            if (FileHandle instanceof FTPSFileStream)
            {
            	((FTPSFileStream)FileHandle).write(Buffer, Index, Size);
                return Size;
            }
            else
            {
                return 0;
            }
        }
        catch (Exception ex)
        {
            return 0;
        }
	}

	@Override
	public int getEntryInformation(String Path, TElVFSEntryInformation EntryInfo) {
        String RealPath = adjustPath(Path);

        if (fileExists(RealPath))
        {
            try
            {
                CopyEntryInfo(new File(RealPath), EntryInfo);
                EntryInfo.setFullName(EntryInfo.getName());
            }
            catch (SecurityException sx)
            {
                return SBCustomFSAdapter.SB_VFS_ERROR_ACCESS_DENIED;
            }
            catch (Exception ex)
            {
                return SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
            }
        }
        else if (directoryExists(RealPath))
        {
            try
            {
                CopyEntryInfo(new File(RealPath), EntryInfo);
                EntryInfo.setFullName(EntryInfo.getName());
            }
            catch (SecurityException sx)
            {
                return SBCustomFSAdapter.SB_VFS_ERROR_ACCESS_DENIED;
            }
            catch (Exception ex)
            {
                return SBCustomFSAdapter.SB_VFS_UNSPECIFIED_ERROR;
            }
        }
        else
        {
            return SBCustomFSAdapter.SB_VFS_ERROR_FILE_NOT_FOUND;
        }

        return 0;
	}

	@Override
	@SuppressWarnings("unused")
	public int getFileStream(String path, int openMode, TSBFileShareMode shareMode,
			TElVFSOutputFileStream strm) {
		Object AHandle = null;

		int res = fileOpen(path, openMode, shareMode, (TElVFSOutputFileHandle) AHandle);
		if ((res == 0) && (AHandle != null))
			strm = (TElVFSOutputFileStream)AHandle;
		else
			strm = null;

		return res;
	}

	@Override
	public TSBInteger readFileTag(String path, String tagName, TSBString value) {
	   TSBInteger resultCode = new TSBInteger();
       OnReadFileTag(this, path, tagName, value, resultCode);
       return resultCode;
	}

	@Override
	public TSBInteger writeFileTag(String path, String name, String value) {
		TSBInteger resultCode = new TSBInteger(); 
		OnWriteFileTag(this, path, name, value, resultCode);
		return resultCode;
	}

	public void OnWriteFileTag(Object Sender, String Path, String TagName,
			String TagValue, TSBInteger resultCode) {
		m_listener.OnWriteFileTag(Sender, Path, TagName, TagValue, resultCode);
	}

	public void OnReadFileTag(Object Sender, String Path, String TagName,
			TSBString TagValue, TSBInteger resultCode) {
	    m_listener.OnReadFileTag(Sender, Path, TagName, TagValue, resultCode);
	}

	public void OnDeleteFileTag(Object Sender, String Path, String TagName, TSBInteger resultCode) {
		m_listener.OnDeleteFileTag(Sender, Path, TagName, resultCode);
	}

}
