import java.awt.Component;
import java.net.URL;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import SecureBlackbox.PGP.SBPGPConstants;
import SecureBlackbox.PGP.TElPGPPublicKey;

public class CustomTreeCellRenderer extends DefaultTreeCellRenderer
{
	private static final long serialVersionUID = 1L;
	
    /** 
    * getTreeCellRendererComponent 
    * This method is overridden to set the node specific icons and tooltips
    *    
    * @return The Component object used to render the cell value
    * @exception 
    */ 
    public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                  boolean selection, boolean expanded,
                                                  boolean leaf, int row, boolean hasFocus)
    {
        super.getTreeCellRendererComponent(tree, value, selection, expanded, 
                                           leaf, row, hasFocus);
        
        //The value object is nothing but the DefaultMutableTreeNode.
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
        
        setIconAndToolTip(node.getUserObject(), tree);
        
        return this;
    } 
    
	private void setIconAndToolTip(Object obj, JTree tree)
    {
    	if (obj instanceof NodeData){
    		NodeData node = (NodeData)obj;
    		  		
    		Object tag = node.getTag();
    		if (tag instanceof TElPGPPublicKey){
    			TElPGPPublicKey key = (TElPGPPublicKey)tag;
    			
				if ((key.getPublicKeyAlgorithm() & (SBPGPConstants.SB_PGP_ALGORITHM_PK_DSA + SBPGPConstants.SB_PGP_ALGORITHM_PK_ELGAMAL)) > 0)
					setIcon(getIcon("0"));
				else
					setIcon(getIcon("1"));		
    		} else if (node.toString().compareTo("signature") == 0)
    			setIcon(getIcon("3"));
    		else setIcon(getIcon("2"));
    	}
    }
	
	private Icon getIcon(String name) {
		URL resPath = getClass().getResource(name + ".ico");
		if (resPath == null) return new ImageIcon();
		return new ImageIcon(resPath);
	}
}