import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JPasswordField;
import javax.swing.ButtonGroup;

import SecureBlackbox.Base.SBConstants;
import SecureBlackbox.Base.TElX509Certificate;
import SecureBlackbox.PDF.TElPDFDocument;
import SecureBlackbox.PDF.TElPDFPasswordSecurityHandler;
import SecureBlackbox.PDF.TElPDFPublicKeySecurityHandler;
import SecureBlackbox.PDF.TElPDFSecurityHandler;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


public class EncOpDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JPasswordField tbPassword;
	private final ButtonGroup buttonGroup = new ButtonGroup();
	private boolean isOk;
	private JRadioButton rbPasswordEncryption;
	private JRadioButton rbPublicKeyEncryption;
	private JCheckBox cbEncryptMetadata;
	private JComboBox<String> cmbEncAlg;

	TElX509Certificate FCert;
	private SelCertDialog frmSelectCertificate;
	private TElPDFPasswordSecurityHandler PasswordHandler;
	private TElPDFPublicKeySecurityHandler PublicKeyHandler;
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			EncOpDialog dialog = new EncOpDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public EncOpDialog() {
		addWindowListener(new WindowAdapter() {
			@Override		
			public void windowClosing(WindowEvent e) {
				if (PublicKeyHandler != null)
					PublicKeyHandler.Destroy();
				if (FCert != null)
					FCert.Destroy();
				if (PasswordHandler != null)
					PasswordHandler.Destroy();
			}	
		});			
		setTitle("Encryption options");
		setBounds(100, 100, 450, 263);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		{
			JPanel panel = new JPanel();
			panel.setBorder(new TitledBorder(null, "", TitledBorder.LEADING, TitledBorder.TOP, null, null));
			panel.setBounds(10, 11, 422, 173);
			contentPanel.add(panel);
			panel.setLayout(null);
			
			cbEncryptMetadata = new JCheckBox("Encrypt document metadata");
			cbEncryptMetadata.setSelected(true);
			cbEncryptMetadata.setBounds(6, 7, 300, 23);
			panel.add(cbEncryptMetadata);
			
			JLabel lblEncryptionAlgorithm = new JLabel("Encryption algorithm:");
			lblEncryptionAlgorithm.setBounds(20, 41, 131, 14);
			panel.add(lblEncryptionAlgorithm);
			
			cmbEncAlg = new JComboBox<String>();
			cmbEncAlg.setModel(new DefaultComboBoxModel<String>(new String[] {"RC4/40 bits (Acrobat 4)", "RC4/128 bits (Acrobat 5)", "AES/128 bits (Acrobat 6, 7)", "AES/256 bits (Acrobat 9)"}));
			cmbEncAlg.setBounds(177, 37, 207, 22);
			panel.add(cmbEncAlg);
			
			rbPasswordEncryption = new JRadioButton("Password encryption");
			rbPasswordEncryption.setSelected(true);
			buttonGroup.add(rbPasswordEncryption);
			rbPasswordEncryption.setBounds(20, 80, 155, 23);
			panel.add(rbPasswordEncryption);
			
			tbPassword = new JPasswordField();
			tbPassword.setBounds(177, 81, 207, 20);
			panel.add(tbPassword);
			
			rbPublicKeyEncryption = new JRadioButton("Public key encryption");
			buttonGroup.add(rbPublicKeyEncryption);
			rbPublicKeyEncryption.setBounds(20, 124, 151, 23);
			panel.add(rbPublicKeyEncryption);
			
			JButton btnSelectCertificate = new JButton("Select certificate");
			btnSelectCertificate.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					frmSelectCertificate.setModal(true);
					frmSelectCertificate.setVisible(true);
					if (frmSelectCertificate.isOK())
					{
						if (FCert != null)
							FCert.Destroy();
						try {
							FCert = frmSelectCertificate.getCert();
						} catch (Exception ex) {
							showErrorMessage(ex);
							ex.printStackTrace();
						}
						if (FCert != null)
							rbPublicKeyEncryption.setSelected(true);
					}
				}
			});
			btnSelectCertificate.setBounds(177, 124, 207, 23);
			panel.add(btnSelectCertificate);
		}
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		init();
	}

	protected void showErrorMessage(Exception ex) {
		JOptionPane.showMessageDialog(null, ex.getMessage(), getTitle(), JOptionPane.ERROR_MESSAGE);	
	}

	void init() {
		frmSelectCertificate = new SelCertDialog();
		PasswordHandler = new TElPDFPasswordSecurityHandler();
		PublicKeyHandler = new TElPDFPublicKeySecurityHandler();
		cmbEncAlg.setSelectedIndex(0);
	}

	public void SetEncryptionHandler(TElPDFDocument Document) throws Exception {
		TElPDFSecurityHandler CurrHandler;
		// setting up property values
		if (rbPasswordEncryption.isSelected())
			CurrHandler = PasswordHandler;
		else
			CurrHandler = PublicKeyHandler;

		// the following encryption handler assignment must be executed before
		// other handler properties are assigned, since encryption handler
		// has to access document properties during its work.
		Document.setEncryptionHandler(CurrHandler);
		CurrHandler.setEncryptMetadata(cbEncryptMetadata.isSelected());
		int Alg, KeySize;
		switch (cmbEncAlg.getSelectedIndex())
		{
			case 0 :
			{
				Alg = SBConstants.SB_ALGORITHM_CNT_RC4;
				KeySize = 40;
				break;
			}

			case 1 :
			{
				Alg = SBConstants.SB_ALGORITHM_CNT_RC4;
				KeySize = 128;
				break;
			}

			case 2 :
			{
				Alg = SBConstants.SB_ALGORITHM_CNT_AES128;
				// the key size for this cipher is always 128 bits, so we may
				// omit the assignment in this point. However, just to calm the
				// compiler we assing the 0 value to KeySize variable.
				KeySize = 0;
				break;
			}

			case 3 :
			{
				Alg = SBConstants.SB_ALGORITHM_CNT_AES256;
				// the key size for this cipher is always 256 bits, so we may
				// omit the assignment in this point. However, just to calm the
				// compiler we assing the 0 value to KeySize variable.
				KeySize = 0;
				break;
			}

			default:
			{
				// normally, we should not reach this point, so just making the
				// compiler silent.
				showErrorMessage("Unsupported algorithm", "Error");
				return;
			}
		}

		// PDF specification allows to use different ciphers for streams and
		// strings contained in a document. However, using the same value for
		// both string and stream encryption allows to achieve greater compatibility
		// with other implementations
		CurrHandler.setStreamEncryptionAlgorithm(Alg);
		CurrHandler.setStringEncryptionAlgorithm(Alg);
		CurrHandler.setStreamEncryptionKeyBits(KeySize);
		CurrHandler.setStringEncryptionKeyBits(KeySize);
		if (CurrHandler instanceof TElPDFPasswordSecurityHandler)
		{
			PasswordHandler.setUserPassword(new String(tbPassword.getPassword()));
			PasswordHandler.setOwnerPassword(new String(tbPassword.getPassword()));
			PasswordHandler.getPermissions().enableAll();
		}
		else
		{
			if (FCert == null)
				throw new Exception("Failed to load encryption certificate");

			// creating recipient group
			int GroupIndex = PublicKeyHandler.addRecipientGroup();
			// adding recipient certificate to group
			PublicKeyHandler.getRecipientGroup(GroupIndex).addRecipient(FCert);
			PublicKeyHandler.getRecipientGroup(GroupIndex).getPermissions().enableAll();
		}
	}

	public boolean isOK() {
		return isOk;
	}

	public boolean getPasswordEncryption() {
		return rbPasswordEncryption.isSelected();
	}

	public TElX509Certificate getCert() {
		return FCert;
	}

	public boolean getEncryptMetadata() {
		return cbEncryptMetadata.isSelected();
	}
	
	void showErrorMessage(String msg, String cap){
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.ERROR_MESSAGE);
	}
}
