

import java.util.Date;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.SFTPClient.*;
import SecureBlackbox.SFTPCommon.*;
import SecureBlackbox.SSHCommon.TElSSHKey;
import SecureBlackbox.SSHCommon.TSSHKeyValidateEvent;


public class Main {

	static TElSimpleSFTPClient Client = null;

	final static int OP_LIST	 = 1;
	final static int OP_DOWNLOAD = 2;
	final static int OP_UPLOAD	 = 3;
	
	static long parOperation = 0;
	static String parRemoteName = "/";
	static String parLocalName;
	static String parUsername;
	static String parPassword;
	static String parAddress;
	static int parPort;
	
	public static void main(String[] args) {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");

		if (!parseCommandLine(args))
			return;
		
		Client = new TElSimpleSFTPClient();
		Client.setAddress(parAddress);
		Client.setPort(parPort);

		Client.setUsername(parUsername);
		Client.setPassword(parPassword);

		Client.setOnKeyValidate(new TSSHKeyValidateEvent(onKeyValidate));

		try
		{
			try
			{
				Client.open();
			}
			catch (Exception e)
			{
				System.out.println("Connection failed due to exception: " + e.getMessage());
				System.out.println("If you have ensured that all connection parameters are correct and you still can't connect,");
				System.out.println("please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php");
				System.out.println("Remember to provide details about the error that happened.");
				if (Client.getServerSoftwareName().length() > 0)
				{
					System.out.println("Server software identified itself as: " + Client.getServerSoftwareName());
				}
				try
				{
					Client.close(true);
				}
				catch (Exception ex)
				{ 
				}
				return;
			}


			try
			{
				if (parOperation == OP_DOWNLOAD) 
					Download();
				else if (parOperation == OP_UPLOAD)
					Upload();
				else if (parOperation == OP_LIST)
					List();
			}
			finally
			{
				Client.close(false);
			}
		}
		catch (Exception e)
		{
			System.out.println("Exception: " + e.getMessage());
		}
	}

	static boolean parseCommandLine(String[] args)
	{
		int i = 0;
		String arg; 

		if (args.length == 0) 
		{
			System.out.println("usage:");
			System.out.println("SFTPClient {-download|-upload} -addr[ess] <address> [-port <port>] -user[name] <user> -pass[word] <password> -remote <remote file name> -local <local file name>");
			return false;
		}
		else
		{
			while (i < args.length)
			{
				arg = args[i];
				arg = arg.toLowerCase();
				if ((arg.compareTo("-down") == 0) || 
					(arg.compareTo("-download") == 0))
				{
					parOperation = OP_DOWNLOAD;
				}
				else
					if ((arg.compareTo("-up") == 0) || 
					(arg.compareTo("-upload") == 0))
				{
					parOperation = OP_UPLOAD;
				}
				else
					if ((arg.compareTo("-list") == 0) || 
					(arg.compareTo("-ls") == 0))
				{
					parOperation = OP_LIST;
				}
				else
					if ((arg.compareTo("-addr") == 0) || 
					(arg.compareTo("-address") == 0))
				{
					i++;
					parAddress = args[i];
				}
				else
					if (arg.compareTo("-port") == 0)
				{
					i++;
					try
					{
						parPort = Integer.parseInt(args[i]);
					}
					catch (Exception ex)
					{
						parPort = 22;
					}
				}
				else
					if ((arg.compareTo("-user") == 0) || 
					(arg.compareTo("-username") == 0))
				{
					i++;
					parUsername = args[i];
				}
				else
					if ((arg.compareTo("-pass") == 0) || 
					(arg.compareTo("-password") == 0))
				{
					i++;
					parPassword = args[i];
				}
				else					
					if (arg.compareTo("-local") == 0)
				{
					i++;
					parLocalName = args[i];
				}
				else					
					if (arg.compareTo("-remote") == 0)
				{
					i++;
					parRemoteName = args[i];
				}
				i = i + 1;
			}
			return (parOperation != 0);
		}
	}

	static void Download()
	{
        
		Client.downloadFile(parRemoteName, parLocalName, TSBFileTransferMode.ftmOverwrite);
	}

	static void Upload()
	{
		Client.uploadFile(parLocalName, parRemoteName, TSBFileTransferMode.ftmOverwrite);
		
		TElSftpFileAttributes Attrs = new TElSftpFileAttributes();
		Attrs.setCTime(new Date());
		Attrs.setMTime(Attrs.getCTime());
		Attrs.setATime(Attrs.getCTime());
		Attrs.setCATime(Attrs.getCTime());

		Attrs.setIncludedAttributes(SBSftpCommon.saATime | SBSftpCommon.saMTime | SBSftpCommon.saCTime | SBSftpCommon.saCATime);
		Client.setAttributes(parRemoteName, Attrs);		
	}

	static void List()
	{
		byte[] Handle;
		ArrayList AListing = new ArrayList();
		Handle = Client.openDirectory(parRemoteName);
		Client.readDirectory(Handle, AListing);
		if (AListing.getCount() > 0)
		{
			System.out.println("Contents of " + parRemoteName + ":");
			for (int i = 0; i < AListing.getCount(); i++)
			{
				System.out.println(((TElSftpFileInfo)AListing.get(i)).getName());
			}
		}
		else
		{
			System.out.println("The specified directory is empty");
		}
		Client.closeHandle(Handle); 
	}

	static TSSHKeyValidateEvent.Callback onKeyValidate = new TSSHKeyValidateEvent.Callback() {
		
		public void tsshKeyValidateEventCallback(TObject sender, TElSSHKey serverKey, TSBBoolean accept) {
			System.out.println("Server key received");
			accept.value = true;
		}
	};
}
