

import java.awt.BorderLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JToolBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.util.Date;

public class Main extends JDialog implements SessionEventListener {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTable tbConnections;
	private JTable tbLog;
	private JButton btnExit;
	private JButton btnStart;
	private JButton btnStop;
	private JButton btnSettings;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			Main dialog = new Main();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				Globals.StopSSHListener();
			}
		});
		setTitle("SSH Server Java demo");
		setBounds(100, 100, 687, 390);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JMenuBar menuBar = new JMenuBar();
		menuBar.setBounds(0, 0, 679, 21);
		contentPanel.add(menuBar);
		{
			JMenu mnServer = new JMenu("Server");
			menuBar.add(mnServer);
			
			JMenuItem mntmStart = new JMenuItem("Start");
			mntmStart.addMouseListener(new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					startServer();
				}
			});
			mnServer.add(mntmStart);
			
			JMenuItem mntmStop = new JMenuItem("Stop");
			mntmStop.addMouseListener(new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					stopServer();
				}
			});
			mnServer.add(mntmStop);
			
			JSeparator separator = new JSeparator();
			mnServer.add(separator);
			
			JMenuItem mntmExit = new JMenuItem("Exit");
			mntmExit.addMouseListener(new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					stopServer();
					dispose();
				}
			});
			mnServer.add(mntmExit);
		}
		
		JMenu mnHelp = new JMenu("Help");
		menuBar.add(mnHelp);
		
		JMenuItem mntmAbout = new JMenuItem("About...");
		mntmAbout.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				AboutDialog d = new AboutDialog();
				d.setModal(true);
				d.setVisible(true);
			}
		});
		mnHelp.add(mntmAbout);
		
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.setBounds(0, 23, 679, 21);
		contentPanel.add(toolBar);
		
		btnStart = new JButton("");
		btnStart.setToolTipText("Start");
		btnStart.setFocusPainted(false);
		btnStart.setBorderPainted(false);
		btnStart.setIcon(getIcon("start"));
		btnStart.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				startServer();
			}
		});
		toolBar.add(btnStart);
		
		btnStop = new JButton("");
		btnStop.setToolTipText("Stop");
		btnStop.setFocusPainted(false);
		btnStop.setBorderPainted(false);
		btnStop.setIcon(getIcon("stop"));
		btnStop.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				stopServer();
			}
		});
		toolBar.add(btnStop);
		
		btnExit = new JButton("");
		btnExit.setToolTipText("Exit");
		btnExit.setFocusPainted(false);
		btnExit.setBorderPainted(false);
		btnExit.setIcon(getIcon("exit"));

		btnExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stopServer();
				dispose();
			}
		});
		btnExit.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				stopServer();
				dispose();
			}
		});
		
		btnSettings = new JButton("");
		btnSettings.setToolTipText("Settings");
		btnSettings.setFocusPainted(false);
		btnSettings.setBorderPainted(false);
		btnSettings.setIcon(getIcon("settings"));
		btnSettings.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				settingsClick();
			}
		});
		toolBar.add(btnSettings);
		toolBar.add(btnExit);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(0, 46, 679, 183);
		contentPanel.add(scrollPane);
		
		tbConnections = new JTable(new ConTableModel());
		tbConnections.setFillsViewportHeight(true);
		scrollPane.setViewportView(tbConnections);
		
		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(0, 230, 679, 133);
		contentPanel.add(scrollPane_1);
		
		tbLog = new JTable(new LogTableModel());
		tbLog.setFillsViewportHeight(true);
		scrollPane_1.setViewportView(tbLog);
		init();
	}

	private Icon getIcon(String name) {
		URL resPath = getClass().getResource(name + ".ico");
		if (resPath == null) return new ImageIcon();
		return new ImageIcon(resPath);
	}

	protected void settingsClick() {
		SettingsDialog.ChangeSettings();
	}

	void init() {
		Globals.initGlobals(this);		
		Globals.setMain(this);
		if (Globals.getSettings().getIsEmpty() == true) 
		{
			showMessage("SSH Server demo", "Welcome to the ElSSHServer Demo Application!\n" + 
				"Thank you for your interest in our products.\n\n" +
				"First of all, you need to create at least one user account.\n\n" + 
				"After clicking the OK button you will be redirected to the\n" +
				"User settings window.");
			SettingsDialog.ChangeSettings();
		}
		Globals.mainRun();
		if (Globals.getServerStarted()) 
		{
			Globals.StopSSHListener();
		}
	}
	
	static void showMessage(String caption, String msg) {
		JOptionPane.showMessageDialog(null, msg, caption, JOptionPane.INFORMATION_MESSAGE);
	}	

	protected void startServer() {
		if (Globals.getSettings().getServerKey().length() == 0)
		{
			showMessage("Info", "Please use Settings button to setup the server and generate a server key");
			return;
		}
		Globals.StartSSHListener();
	}

	protected void stopServer() {
		Globals.StopSSHListener();
	}

	public void addLogEvent(String msg, boolean eFlag) {
		LogTableModel model = (LogTableModel)tbLog.getModel();
		model.addRow(new Date(), msg, eFlag ? "Eror" : "Info");
	}

	public void setTopButtonsEnabled(int i, boolean b) {
		switch (i) {
		case 0:
			btnStart.setEnabled(b);
			break;
		case 1:
			btnStop.setEnabled(b);
		default:
			break;
		}
	}

	public void SessionStarted(SSHSession src) {
        Logger.Log("SSH session started");
	}

	public void SessionClosed(SSHSession session) {
		Logger.Log("SSH session closed");
		int index = (Integer)session.getData();
		ConTableModel model = (ConTableModel)tbConnections.getModel();
		model.removeRowBySessionIndex(index);
	}

	public void SessionInfoChanged(SSHSession session) {
		ConTableModel model = (ConTableModel)tbConnections.getModel();
		if (session.getData() == null){
			int index = tbConnections.getRowCount();
			session.setData(index);
			model.addRow(session);
		} else {
			int index = (Integer) session.getData();
			model.removeRowBySessionIndex(index);
			model.addRow(session);
		}
	}
	
}
