import SecureBlackbox.Base.*;
import SecureBlackbox.SSHCommon.*;

public class UserInfo {

	private String m_Name = "";
	private int m_AuthTypes = 0;
	private String m_PasswordSalt = "";
	private String m_PasswordHash = "";
	private String m_PublicKey = "";
	private boolean m_AuthAll = false;
	private int m_uNumber = 0;
	
	public int getUNumber() {
		return m_uNumber;
	}
	
	public void setUNumber(int value){
		m_uNumber = value;
	}
	
	public String getName(){
		return m_Name;
	}
	
	public void setName(String value){
		m_Name = value;
	}
	
	public int getAuthTypes(){
		return m_AuthTypes;
	}
	
	public void setAuthTypes(int value){
		m_AuthTypes = value;
	}
	
	public String getPasswordSalt(){
		return m_PasswordSalt;
	}
	
	public void setPasswordSalt(String value){
		m_PasswordSalt = value;
	}
	
	public String getPasswordHash() {
		return m_PasswordHash;
	}
	
	public void setPasswordHash(String value){
		m_PasswordHash = value;
	}
	
	public String getPublicKey(){
		return m_PublicKey;
	}
	
	public void setPublicKey(String value){
		m_PublicKey = value;
	}

	public boolean getAuthAll(){
		return m_AuthAll;
	}
	
	public void setAuthAll(boolean value){
		m_AuthAll = value;
	}
	
	public UserInfo() {}
	
	public UserInfo(String UserLine)
	{
		String[] Lexems = UserLine.split(":");
		try
		{
			setName(Lexems[0]);
			setPasswordSalt(Lexems[1]);
			setPasswordHash(Lexems[2]);
			String Auths = Lexems[3];
			int AuthTypes = getAuthTypes();
			if (Auths.indexOf("password") >= 0) 
			{
				AuthTypes |= SBSSHConstants.SSH_AUTH_TYPE_PASSWORD;
			}
			if (Auths.indexOf("publickey") >= 0) 
			{
				AuthTypes |= SBSSHConstants.SSH_AUTH_TYPE_PUBLICKEY;
			}
			if (Auths.indexOf("keyboard") >= 0) 
			{
				AuthTypes |= SBSSHConstants.SSH_AUTH_TYPE_KEYBOARD;
			}
			setAuthTypes(AuthTypes);
			setAuthAll((Auths.indexOf(" all") >= 0));
			if (Lexems.length >= 5)
			{
				TElSSHKey Key = new TElSSHKey();
				byte[] PKey = SBEncoding.base64DecodeArray(Lexems[4]);
				int Result = Key.loadPublicKey(PKey, PKey.length); 
				if (Result == 0) 
				{
					setPublicKey(SBUtils.stringOfBytes(PKey, 0, PKey.length));
				}
			}
		}
		catch(Exception exc) 
		{ 
			Logger.Log("Error loading users " + exc.getMessage(), false);
		}
	}

	public void SetPassword(String Password)
	{
		byte[] Salt = new byte[4];
		SBRandom.sbRndGenerate(Salt, 4);
		String PasswordSalt = toHex(Salt);
		setPasswordSalt(PasswordSalt);
		setPasswordHash(calcPasswordHash(PasswordSalt, Password));
	}

	public static String toHex(byte[] bytes) {
	    java.math.BigInteger bi = new java.math.BigInteger(1, bytes);
	    return String.format("%0" + (bytes.length << 1) + "X", bi);
	}

	private String calcPasswordHash(String Salt, String Password)
	{
		String sHash = Salt + Password;
		byte[] Hash = new byte[sHash.length()];
		for (int i = 0; i < sHash.length(); i++)
			Hash[i] = (byte)sHash.getBytes()[i];
		return SBUtils.digestToStr128(SBMD.hashMD5(Hash), true);
	}

	public boolean PasswordValid(String Password)
	{
		return (getPasswordHash().compareTo(calcPasswordHash(getPasswordSalt(), Password)) == 0);
	}
	
	public boolean SetPublicKey(String Key)
	{
		TElSSHKey TestKey = new TElSSHKey();
		byte[] keyBytes = SBUtils.bytesOfString(Key);
		if (TestKey.loadPublicKey(keyBytes, keyBytes.length) == 0)
		{
			setPublicKey(Key);
			return true;
		} 
		else return false;
	}

	public boolean KeyValid(TElSSHKey Key)
	{
		TElSSHKey CurrentKey = new TElSSHKey();
		try
		{
			byte[] keyBytes = SBUtils.bytesOfString(getPublicKey());
			CurrentKey.loadPublicKey(keyBytes, keyBytes.length);
			byte[] Current = SBUtils.digestToBinary160(CurrentKey.getFingerprintSHA1());
			byte[] Test = SBUtils.digestToBinary160(Key.getFingerprintSHA1());
			for (int i = 0; i < Current.length; i++)
				if (Current[i] != Test[i]) return false;
			return true;
		} 
		catch(Exception ex) 
		{
			return false;
		}
	}

	public String ToString()
	{
		// encoding key to make it a single line:
		byte[] keyBuf = SBUtils.bytesOfString(getPublicKey());
		String encodedKey = "";
		encodedKey = SBEncoding.base64EncodeArray(keyBuf, false);
		String Result = getName() + ":" + getPasswordSalt() + ":" + getPasswordHash() + 
			":" + getAuthType() + ":" + encodedKey;
		return Result;
	}

	public String getAuthType()
	{
		String sAuth = "";
		int AuthTypes = getAuthTypes();
		if ((AuthTypes & SBSSHConstants.SSH_AUTH_TYPE_PASSWORD) != 0)
			sAuth += ",password";
		if ((AuthTypes & SBSSHConstants.SSH_AUTH_TYPE_PUBLICKEY) != 0)
			sAuth += ",publickey";
		if ((AuthTypes & SBSSHConstants.SSH_AUTH_TYPE_KEYBOARD) != 0)
			sAuth += ",keyboard";
		if (getAuthAll()) 
		{
			sAuth += ", all";
		}
		if (sAuth.length() > 0) 
		{
			StringBuffer s = new StringBuffer(sAuth);
			s.delete(0, 1);
			sAuth = s.toString();
		}
		return sAuth;
	}
}
