import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.JTable;
import javax.swing.JScrollPane;

import SecureBlackbox.Base.TSBCertificateValidity;
import SecureBlackbox.XMLSecurity.TElXAdESVerifier;
import SecureBlackbox.XMLSecurity.TElXMLSOAPBaseSignatureHandler;
import SecureBlackbox.XMLSecurity.TSBXMLSOAPSignatureValidationStatus;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


public class SignatureHandlerInfoFrm extends JDialog {

	private final JPanel contentPanel = new JPanel();
	private JTable table;

	/**
	 * Create the dialog.
	 */
	public SignatureHandlerInfoFrm() {
		setTitle("Signature Handler Info");
		setBounds(100, 100, 505, 388);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(10, 10, 469, 296);
		contentPanel.add(scrollPane);
		
		table = new JTable(new SignatureHandlerInfoModel());
		table.setFillsViewportHeight(true);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scrollPane.setViewportView(table);
		
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setVisible(false);
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
	}
	
	public void init(TElXMLSOAPBaseSignatureHandler handler, TSBXMLSOAPSignatureValidationStatus validationStatus,
			TSBCertificateValidity certValidity, int certReason) 
	{
		SignatureHandlerInfoModel model = (SignatureHandlerInfoModel) table.getModel();
		model.clear();
		
		if (validationStatus == TSBXMLSOAPSignatureValidationStatus.ssvsValid) {
			model.add("Signature", "Valid");
			
			if (handler.getSignerCertificate() != null) {
				if (certValidity == TSBCertificateValidity.cvSelfSigned) {
					model.add("Certificate Validation", "The selected signature is signed by self-signed certificate which was not previously trusted");
				} else if (certValidity != TSBCertificateValidity.cvOk) {
					String s;
					
					if (certValidity == TSBCertificateValidity.cvChainUnvalidated)
						s = "Signing certificate chain could not be validated completely";
					else
						s = "Signing certificate is not valid";
					
					model.add("Certificate Validation", s);
				} else {
					model.add("Certificate Validation", "Signing certificate validated successfully");
				}
				
				if (certReason != 0)
					model.add("Certificate Validation Reason", Utils.x509ReasonToString(certReason));
			}
		} else {
			String s;
			
			if (validationStatus == TSBXMLSOAPSignatureValidationStatus.ssvsInvalidSignature)
				s = "Signature is INVALID";
			else if (validationStatus == TSBXMLSOAPSignatureValidationStatus.ssvsNoKey)
				s = "No Key specified";
			else if (validationStatus == TSBXMLSOAPSignatureValidationStatus.ssvsInvalidReferenceDigest)
				s = "Signature contains an invalid references";
			else
				s = "Unknown validation status";
			
			model.add("Validation Status", s);
			model.add("Signature", "INVALID");
		}
		
		model.add("", "");
		
		if ((handler.getXAdESProcessor() instanceof TElXAdESVerifier) && ((TElXAdESVerifier)handler.getXAdESProcessor()).getIsEnabled())
			model.add("XAdES", "Present");	
	}
}
